#include <common/types.h>
#include <common/multitasking.h>
#include <common/memory.h>
#include <hardwarecommunication/gdt.h>
#include <hardwarecommunication/interrupts.h>
#include <hardwarecommunication/pci.h>
#include <drivers/drivers.h>
#include <drivers/keyboard.h>
#include <drivers/mouse.h>
#include <drivers/io.h>

typedef void (*constructor)();
extern "C" constructor start_ctors;
extern "C" constructor end_ctors;

extern "C" void callConstructors() {
    for (constructor* i = &start_ctors; i != &end_ctors; i++) {
        (*i)();
    }
}

void TKSKOSPowerShell(){
    printf("TKSKOS PowerShell.\n");
    printf("[WARNING] TKSKOS PowerShell Is Developing...\n\n");
    while(1);
    return;
}

extern "C" void kernelMain(void* multiboot_structure, uint32_t magicnumber) {
    GlobalDescriptorTable gdt;

    uint32_t* memupper = (uint32_t*)(((size_t)multiboot_structure) + 8);
    size_t heap = 10 * 1024 * 1024;
    MemoryManager memoryManager(heap, (*memupper) * 1024 - heap - 10 * 1024);

    TaskManager taskManager;

    InterruptManager interrupts(0x20, &gdt, &taskManager);

    KeyBoardDriver keyboard(&interrupts);
    //MouseDriver mouse(&interrupts);
    DriversManager drvmanager;
    drvmanager.AddDrivers(&keyboard);
    //drvmanager.AddDrivers(&mouse);
    PCIController PCIController;
    PCIController.SelectDrivers(&drvmanager, &interrupts);
    drvmanager.AllActivate();

    interrupts.Activate();

    printf("[INFO] TKSKOS's Kernel Loaded.\n");
    printf("[INFO] Welcome To TKSKOS!\n");
    printf("[INFO] Maker : Tokisakix.\n\n");

    TKSKOSPowerShell();

    return;
}